import { InjectionToken, Inject, Injectable, NgModule, Optional, SkipSelf } from '@angular/core';
import { Title, Meta } from '@angular/platform-browser';
import { Router, ActivatedRoute, RouterModule } from '@angular/router';
import 'rxjs/add/operator/filter';
import 'rxjs/add/operator/map';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const META_CONFIG_TOKEN = new InjectionToken('meta config');
const META_GUARD_IDENTIFIER = 'MetaGuard';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const isDefined = (val) => typeof val !== 'undefined';
class MetaService {
    /**
     * @param {?} router
     * @param {?} meta
     * @param {?} titleService
     * @param {?} activatedRoute
     * @param {?} metaConfig
     */
    constructor(router, meta, titleService, activatedRoute, metaConfig) {
        this.router = router;
        this.meta = meta;
        this.titleService = titleService;
        this.activatedRoute = activatedRoute;
        this.metaConfig = metaConfig;
        this._warnMissingGuard();
    }
    /**
     * @param {?=} title
     * @param {?=} titleSuffix
     * @return {?}
     */
    setTitle(title, titleSuffix) {
        let /** @type {?} */ titleContent = isDefined(title) ? title : (this.metaConfig.defaults['title'] || '');
        if (this.metaConfig.useTitleSuffix) {
            titleContent += isDefined(titleSuffix) ? titleSuffix : (this.metaConfig.defaults['titleSuffix'] || '');
        }
        this._updateMetaTag('title', titleContent);
        this._updateMetaTag('og:title', titleContent);
        this.titleService.setTitle(titleContent);
        return this;
    }
    /**
     * @param {?} tag
     * @param {?} value
     * @return {?}
     */
    setTag(tag, value) {
        if (tag === 'title' || tag === 'titleSuffix') {
            throw new Error(`Attempt to set ${tag} through 'setTag': 'title' and 'titleSuffix' are reserved tag names.
      Please use 'MetaService.setTitle' instead`);
        }
        let /** @type {?} */ content = isDefined(value) ? value : (this.metaConfig.defaults[tag] || '');
        this._updateMetaTag(tag, content);
        if (tag === 'description') {
            this._updateMetaTag('og:description', content);
        }
        return this;
    }
    /**
     * @param {?} tag
     * @param {?} value
     * @return {?}
     */
    _updateMetaTag(tag, value) {
        let /** @type {?} */ prop = 'name';
        if (tag.startsWith(`og:`)) {
            prop = 'property';
        }
        this.meta.updateTag({
            [prop]: tag,
            content: value
        });
    }
    /**
     * @return {?}
     */
    _warnMissingGuard() {
        if (isDefined(this.metaConfig.warnMissingGuard) && !this.metaConfig.warnMissingGuard) {
            return;
        }
        const /** @type {?} */ hasDefaultMeta = !!Object.keys(this.metaConfig.defaults).length;
        const /** @type {?} */ hasMetaGuardInArr = (it) => (it && it.IDENTIFIER === META_GUARD_IDENTIFIER);
        let /** @type {?} */ hasShownWarnings = false;
        this.router.config.forEach((route) => {
            const /** @type {?} */ hasRouteMeta = route.data && route.data["meta"];
            const /** @type {?} */ showWarning = !isDefined(route.redirectTo)
                && (hasDefaultMeta || hasRouteMeta)
                && !(route.canActivate || []).some(hasMetaGuardInArr);
            if (showWarning) {
                console.warn(`Route with path "${route.path}" has ${hasRouteMeta ? '' : 'default '}meta tags, but does not use MetaGuard. \
Please add MetaGuard to the canActivate array in your route configuration`);
                hasShownWarnings = true;
            }
        });
        if (hasShownWarnings) {
            console.warn(`To disable these warnings, set metaConfig.warnMissingGuard: false \
in your ng2-meta MetaConfig passed to MetaModule.forRoot()`);
        }
    }
}
MetaService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
MetaService.ctorParameters = () => [
    { type: Router, },
    { type: Meta, },
    { type: Title, },
    { type: ActivatedRoute, },
    { type: undefined, decorators: [{ type: Inject, args: [META_CONFIG_TOKEN,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const DEFAULT_META_CONFIG = {
    useTitleSuffix: false,
    warnMissingGuard: true,
    defaults: {}
};
/**
 * Additional site-wide configuration of meta tags.
 *
 * Use this to enable adding a title suffix to the title of each route,
 * or to set default meta tags to be used when no equivalent meta tag
 * exists in the meta configuration of the route.
 *
 *
 * Default
 * ```
 * {
 *   useTitleSuffix: false,
 *   defaults: {}
 * }
 * ```
 * @record
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MetaGuard {
    /**
     * @param {?} router
     * @param {?} metaService
     * @param {?} metaConfig
     */
    constructor(router, metaService, metaConfig) {
        this.router = router;
        this.metaService = metaService;
        this.metaConfig = metaConfig;
    }
    /**
     * @param {?} route
     * @param {?} state
     * @return {?}
     */
    canActivate(route, state) {
        this._processRouteMetaTags(route.data && route.data["meta"]);
        return true;
    }
    /**
     * @param {?=} meta
     * @return {?}
     */
    _processRouteMetaTags(meta = {}) {
        if (meta.disableUpdate) {
            return;
        }
        this.metaService.setTitle(meta.title, meta.titleSuffix);
        Object.keys(meta).forEach(key => {
            if (key === 'title' || key === 'titleSuffix') {
                return;
            }
            this.metaService.setTag(key, meta[key]);
        });
        Object.keys(this.metaConfig.defaults).forEach(key => {
            if (key in meta || key === 'title' || key === 'titleSuffix') {
                return;
            }
            this.metaService.setTag(key, this.metaConfig.defaults[key]);
        });
    }
}
MetaGuard.IDENTIFIER = META_GUARD_IDENTIFIER;
MetaGuard.decorators = [
    { type: Injectable },
];
/** @nocollapse */
MetaGuard.ctorParameters = () => [
    { type: Router, },
    { type: MetaService, },
    { type: undefined, decorators: [{ type: Inject, args: [META_CONFIG_TOKEN,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MetaModule {
    /**
     * @param {?} parentModule
     */
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('MetaModule is already loaded. Import it in the AppModule only');
        }
    }
    /**
     * @param {?=} metaConfig
     * @return {?}
     */
    static forRoot(metaConfig = DEFAULT_META_CONFIG) {
        return {
            ngModule: MetaModule,
            providers: [
                { provide: META_CONFIG_TOKEN, useValue: metaConfig },
                MetaGuard,
                MetaService
            ]
        };
    }
}
MetaModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    RouterModule
                ]
            },] },
];
/** @nocollapse */
MetaModule.ctorParameters = () => [
    { type: MetaModule, decorators: [{ type: Optional }, { type: SkipSelf },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { MetaService, MetaModule, MetaGuard, DEFAULT_META_CONFIG as ɵa, META_CONFIG_TOKEN as ɵb };
//# sourceMappingURL=ng2-meta.js.map
